"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = actualNameHandler;

var _astTypes = require("ast-types");

var _reactDocgen = require("react-docgen");

/*
 * Copyright (c) 2015, Facebook, Inc.
 * All rights reserved.
 *
 * This source code is licensed under the BSD-style license found in the
 * LICENSE file in the root directory of this source tree. An additional grant
 * of patent rights can be found in the PATENTS file in the same directory.
 *
 *
 * 2018 - Charles Lehnert
 * This is heavily based on the react-docgen `displayNameHandler` but instead defines an `actualName` property on the
 * generated docs that is taken first from the component's actual name. This addresses an issue where the name that
 * the generated docs are stored under is incorrectly named with the `displayName` and not the component's actual name.
 *
 */
var getNameOrValue = _reactDocgen.utils.getNameOrValue,
    isReactForwardRefCall = _reactDocgen.utils.isReactForwardRefCall;

function actualNameHandler(documentation, path, importer) {
  // Function and class declarations need special treatment. The name of the
  // function / class is the displayName
  if (_astTypes.namedTypes.ClassDeclaration.check(path.node) || _astTypes.namedTypes.FunctionDeclaration.check(path.node)) {
    documentation.set('actualName', getNameOrValue(path.get('id')));
  } else if (_astTypes.namedTypes.ArrowFunctionExpression.check(path.node) || _astTypes.namedTypes.FunctionExpression.check(path.node) || isReactForwardRefCall(path, importer)) {
    var currentPath = path;

    while (currentPath.parent) {
      if (_astTypes.namedTypes.VariableDeclarator.check(currentPath.parent.node)) {
        documentation.set('actualName', getNameOrValue(currentPath.parent.get('id')));
        return;
      } else if (_astTypes.namedTypes.AssignmentExpression.check(currentPath.parent.node)) {
        var leftPath = currentPath.parent.get('left');

        if (_astTypes.namedTypes.Identifier.check(leftPath.node) || _astTypes.namedTypes.Literal.check(leftPath.node)) {
          documentation.set('actualName', getNameOrValue(leftPath));
          return;
        }
      }

      currentPath = currentPath.parent;
    }
  } else if ( // React.createClass() or createReactClass()
  _astTypes.namedTypes.CallExpression.check(path.parentPath.node) && _astTypes.namedTypes.VariableDeclarator.check(path.parentPath.parentPath.parentPath.node)) {
    documentation.set('actualName', getNameOrValue(path.parentPath.parentPath.parentPath.get('id')));
  } else {
    // Could not find an actual name
    documentation.set('actualName', '');
  }
}